@@ -21,7 +21,10 @@ int main(int argc, char* argv[])
         if (Error) goto E0;
     }
 
-E0: printf("\n%s%d\n", "Error: ", Error);
+E0:
+    #if (_REBMIXEXE)
+    printf("\n%s%d\n", "Error: ", Error);
+    #endif
     
     return (Error);
 } /* main */
